/*
 * Copyright (c) 2003, Raymond Bosman
 * Copyright (c) 2003, Frederik Holljen
 * All Rights Reserved.
 *
 * See COPYING for licensing.
 */


//! This is the runtime library for the Ck language
/*!
 * Currently it contains the functions needed for implementing the advanced array operations
 * ==, != and =.
 */

//! Check if two single dimension arrays of type int or bool are the same.
bool sameIntSingle( int a1[], int a2[], int length )
{
    int i = 0;
    bool ret = true;
    while( i < length && ret )
    {
        if( a1[i] != a2[i] )
        {
            ret = false;
        };
        i = i + 1;
    };
    return ret;
}

//! Check if two single dimension arrays of type int or bool are dissimilar.
bool nSameIntSingle(  int a1[], int a2[], int length )
{
    return !(sameIntSingle( a1, a2, length ));
}

//! Check if two single dimension arrays of type char are the same.
bool sameCharSingle( char a1[], char a2[], int length )
{
    int i = 0;
    bool ret = true;
    while( i < length && ret )
    {
        if( a1[i] != a2[i] )
        {
            ret = false;
        };
        i = i + 1;
    };
    return ret;
}

//! Check if two single dimension arrays of type char are dissimilar.
bool nSameCharSingle(  char a1[], char a2[], int length )
{
    return !(sameCharSingle( a1, a2, length ));
}

// DUAL arrays
//! Check if two double dimension arrays of type int or bool are the same.
bool sameIntDouble( int a1[][], int a2[][], int length1, int length2 )
{
    int i = 0;
    bool ret = true;
    while( i < length1 && ret )
    {
        if( !sameIntSingle( a1[i], a2[i], length2 ) )
        {
            ret = false;
        };
        i = i + 1;
    };
    return ret;
}

//! Check if two double dimension arrays of type int or bool are dissimilar.
bool nSameIntDouble( int a1[][], int a2[][], int length1, int length2 )
{
    return !(sameIntDouble( a1, a2, length1, length2));
}

//! Check if two double dimension arrays of type char are the same.
bool sameCharDouble( char a1[][], char a2[][], int length1, int length2 )
{
    int i = 0;
    bool ret = true;
    while( i < length1 && ret )
    {
        if( !sameCharSingle( a1[i], a2[i], length2 ) )
        {
            ret = false;
        };
        i = i + 1;
    };
    return ret;
}

//! Check if two double dimension arrays of type char are dissimilar.
bool nSameCharDouble( char a1[][], char a2[][], int length1, int length2 )
{
    return !(sameCharDouble( a1, a2, length1, length2));
}

//! Copy integer source to target for length units in single array.
void singleIntAssign( int target[], int source[], int length )
{
    int i = 0;
    while( i < length )
    {
        target[i] = source[i];
        i = i + 1;
    };
    return 0;
}

//! Copy integer source to target for length1 and length2 units in double array.
void doubleIntAssign( int target[][], int source[][], int length1, int length2 )
{
    int i = 0;
    while( i < length1 )
    {
        singleIntAssign( target[i], source[i], length2 );
        i = i + 1;
    };
    return 0;
}

//! Copy char source to target for length units in single array.
void singleCharAssign( char target[], char source[], int length )
{
    int i = 0;
    while( i < length )
    {
        target[i] = source[i];
        i = i + 1;
    };
    return 0;
}

//! Copy char source to target for length1 and length2 units in double array.
void doubleCharAssign( char target[][], char source[][], int length1, int length2 )
{
    int i = 0;
    while( i < length1 )
    {
        singleCharAssign( target[i], source[i], length2 );
        i = i + 1;
    };
    return 0;
}

// we need a main or Ck will get sick
int main()
{
    int a[2] = { 1,2 };
    int b[2] = { 1,2 };
    int c[2];
    put( sameIntSingle( a,b,2 ) );
    singleIntAssign( c, b, 2 );
    put( sameIntSingle( a,c,2 ) );
    return 0;
}
